const execSync = require('child_process').execSync;
const fs = require('fs');
const path = require('path');
const renameRegisterComponent = require('./rename');

/**
 * Use Yarn if available, it's much faster than the npm client.
 * Return the version of yarn installed on the system, null if yarn is not available.
 */
function getYarnVersionIfAvailable() {
  let yarnVersion;
  try {
    // execSync returns a Buffer -> convert to string
    yarnVersion = (
      execSync('yarn --version', {
        stdio: [0, 'pipe', 'ignore'],
      }).toString() || ''
    ).trim();
  } catch (error) {
    return null;
  }
  return yarnVersion;
}

function getDetoxVersionIfAvailable() {
  let detoxVersion;
  try {
    // execSync returns a Buffer -> convert to string
    detoxVersion = (
      execSync('detox --version', {
        stdio: [0, 'pipe', 'ignore'],
      }).toString() || ''
    ).trim();
  } catch (error) {
    return null;
  }
  return detoxVersion;
}

function addAdditionalScripts() {
  const scriptsJsonPath = path.resolve('scriptsDependencies.json');
  fs.stat(scriptsJsonPath, (error) => {
    if (error) return;
    console.log('Adding scripts to package.json');
    const fileName = 'package.json';
    const packageFile = path.resolve(fileName);
    let file = require(packageFile);

    const scripts = JSON.parse(fs.readFileSync(scriptsJsonPath));

    // these are the scripts that will be added to package.json
    for (const scriptName in scripts) {
      const script = scripts[scriptName];
      file.scripts[scriptName] = script;
    }

    delete file.jest;

    file.detox = {
      configurations: {
        'ios.sim.debug': {
          binaryPath: `ios/build/Build/Products/Debug-iphonesimulator/${file.name}.app`,
          build: `xcodebuild -workspace ios/${file.name}.xcworkspace -scheme ${file.name} -configuration Debug -sdk iphonesimulator -derivedDataPath ios/build`,
          type: 'ios.simulator',
          device: {
            type: 'iPhone 11',
          },
        },
      },
      'test-runner': 'jest',
    };

    fs.writeFileSync(fileName, JSON.stringify(file, null, 2));
    console.log('Deleting scriptsDependencies.json...');
    execSync(`rm -rf ${scriptsJsonPath}`);
  });
}

function installDependencies() {
  const dependenciesJsonPath = path.resolve('dependencies.json');
  fs.stat(dependenciesJsonPath, (error) => {
    if (error) return;
    console.log('Adding dependencies for the project...');
    const dependencies = JSON.parse(fs.readFileSync(dependenciesJsonPath));
    let depsToInstall = [];

    for (const depName in dependencies) {
      const depVersion = dependencies[depName];
      const depToInstall = `${depName}@${depVersion}`;
      depsToInstall.push(depToInstall);
    }

    depsToInstall = depsToInstall.join(' ');
    console.log(`Adding ${depsToInstall}...`);
    if (getYarnVersionIfAvailable()) {
      execSync(`yarn add ${depsToInstall}`, { stdio: 'inherit' });
    } else {
      execSync(`npm install ${depsToInstall} --save`);
    }
    console.log('Deleting dependencies.json...');
    execSync(`rm -rf ${dependenciesJsonPath}`);
  });
}

function installDevDependencies() {
  const devDependenciesJsonPath = path.resolve('devDependencies.json');
  fs.stat(devDependenciesJsonPath, (error) => {
    if (error) return;
    console.log('Adding dev dependencies for the project...');
    const devDependencies = JSON.parse(fs.readFileSync(devDependenciesJsonPath));
    let depsToInstall = [];

    for (const depName in devDependencies) {
      const depVersion = devDependencies[depName];
      const depToInstall = `${depName}@${depVersion}`;
      depsToInstall.push(depToInstall);
    }

    depsToInstall = depsToInstall.join(' ');
    console.log(`Adding ${depsToInstall}...`);
    if (getYarnVersionIfAvailable()) {
      execSync(`yarn add ${depsToInstall} -D`, { stdio: 'inherit' });
    } else {
      execSync(`npm install ${depsToInstall} --save-dev`);
    }
    console.log('Deleting devDependencies.json...');
    execSync(`rm -rf ${devDependenciesJsonPath}`);
  });
}

function addE2ETest() {
  const fileName = 'package.json';
  const packageFile = path.resolve(fileName);
  let file = require(packageFile);

  file.detox = {
    configurations: {
      'ios.sim.debug': {
        binaryPath: `ios/build/Build/Products/Debug-iphonesimulator/${file.name}.app`,
        build: `xcodebuild -workspace ios/${file.name}.xcworkspace -scheme ${file.name} -configuration Debug -sdk iphonesimulator -derivedDataPath ios/build`,
        type: 'ios.simulator',
        device: {
          type: 'iPhone 11',
        },
      },
      'android.emu.debug': {
        binaryPath: 'android/app/build/outputs/apk/debug/app-debug.apk',
        build: 'cd android && ./gradlew assembleDebug assembleAndroidTest -DtestBuildType=debug && cd ..',
        type: 'android.emulator',
        device: {
          avdName: 'Pixel_XL_API_26',
        },
      },
    },
    'test-runner': 'jest',
  };

  fs.writeFileSync(packageFile, JSON.stringify(file, null, 2));
}

function replaceDependencies() {
  replaceReactNavigationBackgroundColor(
    'node_modules/react-navigation-stack/lib/commonjs/views/StackView/StackViewTransitionConfigs.js',
  );
  replaceReactNavigationBackgroundColor(
    'node_modules/react-navigation-stack/lib/module/views/StackView/StackViewTransitionConfigs.js',
  );

  replaceAndroidSupportPath(
    'node_modules/@react-native-community/viewpager/android/src/main/java/com/reactnativecommunity/viewpager/ReactViewPager.java',
  );
  replaceAndroidSupportPath(
    'node_modules/@react-native-community/viewpager/android/src/main/java/com/reactnativecommunity/viewpager/ReactViewPagerManager.java',
  );
}

function replaceReactNavigationBackgroundColor(params) {
  const reactNavigationPath = path.resolve(params);
  fs.stat(reactNavigationPath, (error) => {
    if (error) return;
    const content = fs.readFileSync(reactNavigationPath, 'utf-8');
    const newContent = content.replace(/#eee/g, 'transparent');
    fs.writeFileSync(reactNavigationPath, newContent);
  });
}

function replaceAndroidSupportPath(params) {
  const filePath = path.resolve(params);
  fs.stat(filePath, (error) => {
    if (error) return;
    const content = fs.readFileSync(filePath, 'utf-8');
    const newContent = content.replace(/android.support.v4.view/g, 'androidx.viewpager.widget');
    fs.writeFileSync(filePath, newContent);
  });
}

function clearNodeModulesFile() {
  const nodeModulesFilePath = path.resolve('node_modules/react-bloc/node_modules/react');
  execSync(`rm -rf ${nodeModulesFilePath}`);
}

function clearScriptsDir() {
  const scriptsDirPath = path.resolve('scripts');
  execSync(`rm -rf ${scriptsDirPath}`);
}

function clearRNDefaultFile() {
  const appFile = path.resolve('App.js');
  execSync(`rm -rf ${appFile}`);
  const appTestFile = path.resolve('__tests__/App-test.js');
  execSync(`rm -rf ${appTestFile}`);
  execSync(`rm -rf .prettierrc.js`);
}

function clearE2EDir() {
  if (getDetoxVersionIfAvailable()) {
    execSync(`rm -rf e2e`);
  }
}

function generateE2EDir() {
  if (getDetoxVersionIfAvailable()) {
    clearE2EDir();
    execSync('detox init -r jest');
  }
}

renameRegisterComponent(() => {
  addAdditionalScripts();
  installDependencies();
  installDevDependencies();
  replaceDependencies();
  addE2ETest();
  clearNodeModulesFile();
  clearScriptsDir();
  clearRNDefaultFile();
  generateE2EDir();
});
